@@ -16,7 +16,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 
 #include <gtk/gtk.h>
@@ -147,7 +147,7 @@ static GTree *canvas_nodes = NULL;      /* We don't use the nodes tree directly
                                  * separate data from presentation: that is, we need to
                                  * keep a list of CanvasItems, but we do not want to keep
                                  * that info on the nodes tree itself */
-static GTree *canvas_links;     /* See canvas_nodes */
+static GTree *canvas_links = NULL;     /* See canvas_nodes */
 static guint known_protocols = 0;
 static canvas_background_t canvas_background;
 static guint displayed_nodes;
@@ -158,6 +158,7 @@ static gint diagram_timeout = 0;        /* Descriptor of the diagram timeout fun
 
 static long canvas_obj_count = 0; /* counter of canvas objects */
 static GooCanvas *gcanvas_ = NULL; /* drawing canvas */
+static GtkContainer *garea_ = NULL; /* drawing container */
 static GooCanvasText *pcap_stats_text_item = NULL;
 
 /***************************************************************************
@@ -165,7 +166,7 @@ static GooCanvasText *pcap_stats_text_item = NULL;
  * local Function definitions
  *
  **************************************************************************/
-static gboolean update_diagram(GooCanvas *canvas); /* full diagram update */
+static void update_diagram(GooCanvas *canvas); /* full diagram update */
 static void diagram_update_nodes(GooCanvas *canvas); /* updates ALL nodes */
 static void diagram_update_links(GooCanvas *canvas); /* updates ALL links */
 static void diagram_update_background_image(GooCanvas *canvas); /* updates background image */
@@ -290,7 +291,6 @@ static void addref_canvas_obj(GObject *obj)
  * signals to callback functions */
 void init_diagram(GtkBuilder *xml)
 {
-  GtkContainer *area;
   GooCanvasItem *rootitem;
   GooCanvasItem *item;
   GtkAllocation windowsize;
@@ -302,9 +302,9 @@ void init_diagram(GtkBuilder *xml)
   g_assert(gcanvas_ == NULL);
 
   /* get containing window and size */
-  area = GTK_CONTAINER(gtk_builder_get_object(xml, "diagramwindow"));
-  g_assert(area != NULL);
-  gtk_widget_get_allocation(GTK_WIDGET(area), &windowsize);
+  garea_ = GTK_CONTAINER(gtk_builder_get_object(xml, "diagramwindow"));
+  g_assert(garea_ != NULL);
+  gtk_widget_get_allocation(GTK_WIDGET(garea_), &windowsize);
 
   /* Creates trees */
   canvas_nodes = g_tree_new_full((GCompareDataFunc)canvas_node_compare,
@@ -328,7 +328,7 @@ void init_diagram(GtkBuilder *xml)
 
   gtk_widget_show(GTK_WIDGET(gcanvas_));
 
-  gtk_container_add(area, GTK_WIDGET(gcanvas_));
+  gtk_container_add(garea_, GTK_WIDGET(gcanvas_));
 
   rootitem = goo_canvas_get_root_item(gcanvas_);
 
@@ -362,7 +362,7 @@ void init_diagram(GtkBuilder *xml)
   pcap_stats_text_item = GOO_CANVAS_TEXT(item);
   addref_canvas_obj(G_OBJECT(pcap_stats_text_item));
 
-  sig_id = g_signal_connect(G_OBJECT(area), "size-allocate",
+  sig_id = g_signal_connect(G_OBJECT(garea_), "size-allocate",
                             G_CALLBACK(diagram_resize_event), gcanvas_);
   g_assert(sig_id > 0);
   sig_id = g_signal_connect(G_OBJECT(pcap_stats_text_item), "enter-notify-event",
@@ -378,7 +378,15 @@ void init_diagram(GtkBuilder *xml)
   /* Set the already_updating global flag */
   already_updating = FALSE;
   stop_requested = FALSE;
-}                               /* init_diagram */
+}
+
+void cleanup_diagram()
+{
+  if (canvas_nodes)
+    g_tree_destroy(canvas_nodes);
+  if (canvas_links)
+    g_tree_destroy(canvas_links);
+}
 
 /*
  * Initialize the canvas_background structure.
@@ -478,6 +486,7 @@ static gboolean diagram_resize_event(GtkWidget *widget,
 
   redraw_canvas_background(canvas);
   diagram_reposition(canvas);
+  diagram_update_links(canvas);
   return FALSE;
 }
 
@@ -524,13 +533,6 @@ static void diagram_update_nodes(GooCanvas *canvas)
   GList *delete_list = NULL;
   node_t *new_node = NULL;
 
-  /* Deletes all nodes and updates traffic values */
-  /* TODO To reduce CPU usage, I could just as well update each specific
-   * node in update_canvas_nodes and create a new timeout function that would
-   * make sure that old nodes get deleted by calling update_nodes, but
-   * not as often as with diagram_refresh_period */
-  nodes_catalog_update_all();
-
   /* Check if there are any new nodes */
   while ((new_node = new_nodes_pop()))
     check_new_node(new_node, canvas);
@@ -560,6 +562,10 @@ static void diagram_update_nodes(GooCanvas *canvas)
 static void diagram_reposition(GooCanvas *canvas)
 {
   reposition_node_t rdata;
+
+  if (pref.headless)
+    return;
+
   init_reposition(&rdata, canvas, displayed_nodes);
 
   g_tree_foreach(canvas_nodes,
@@ -583,8 +589,8 @@ static void diagram_update_links(GooCanvas *canvas)
 {
   GList *delete_list = NULL;
 
-  /* Delete old capture links and update capture link variables */
-  links_catalog_update_all();
+  if (pref.headless)
+    return;
 
   /* Check if there are any new links */
   links_catalog_foreach((GTraverseFunc)check_new_link, canvas);
@@ -685,27 +691,36 @@ static void update_pcap_stats_text(GooCanvas *canvas)
  * 2. Updates nodes looks
  * 3. Updates links looks
  */
-static gboolean update_diagram(GooCanvas *canvas)
+static void update_diagram(GooCanvas *canvas)
 {
-  static struct timeval last_refresh_time = {
-    0, 0
-  };
   capstatus_t status;
 
   /* if requested and enabled, dump to xml */
   if (appdata.request_dump && appdata.export_file_signal) {
     g_warning(_("SIGUSR1 received: exporting to %s"), appdata.export_file_signal);
-    dump_xml(appdata.export_file_signal);
+    dump_xml(appdata.export_file_signal, appdata.n_packets);
     appdata.request_dump = FALSE;
   }
 
   status = get_capture_status();
   if (status == PAUSE)
-    return FALSE;
+    return;
 
   if (status == CAP_EOF) {
     gui_eof_capture();
-    return FALSE;
+
+    if (pref.headless)
+      gtk_main_quit();
+
+#if DEBUG_TIMINGS
+    /* after capture stops, update the time in 100ms steps for debugging */
+    static struct timeval incr;
+    incr.tv_sec = 0;
+    incr.tv_usec = 100000;
+    timeradd(&appdata.now, &incr, &appdata.now);
+#else
+    return;  /* after replaying do not update the display anymore */
+#endif        
   }
 
   /*
@@ -718,52 +733,53 @@ static gboolean update_diagram(GooCanvas *canvas)
 
   if (already_updating) {
     g_my_debug("update_diagram called while already updating");
-    return FALSE;
+    return;
   }
 
   already_updating = TRUE;
-  gettimeofday(&appdata.gui_now, NULL);
 
-  /* update nodes */
-  diagram_update_nodes(canvas);
+  if (!pref.headless || appdata.max_delay) {
+    /* headless with max delay zero disables also node expiration */ 
 
-  /* update links */
-  diagram_update_links(canvas);
+    /* Deletes all nodes and updates traffic values */
+    nodes_catalog_update_all();
 
-  /* update background image */
-  diagram_update_background_image(canvas);
+    /* Delete old capture links and update capture link variables */
+    links_catalog_update_all();
 
-  /* Update protocol information */
-  protocol_summary_update_all();
+    /* Update global protocol information */
+    protocol_summary_update_all();
+  }
 
-  /* update proto legend */
-  update_legend();
+  if (!pref.headless) {
 
-  /* Now update info windows */
-  update_info_windows(NULL);
+    /* update background image */
+    diagram_update_background_image(canvas);
 
-  update_pcap_stats_text(canvas);
+    /* update nodes */
+    diagram_update_nodes(canvas);
 
-  /* With this we make sure that we don't overload the
-   * CPU with redraws */
+    /* update links */
+    diagram_update_links(canvas);
 
-  if ((last_refresh_time.tv_sec == 0) && (last_refresh_time.tv_usec == 0))
-    last_refresh_time = appdata.gui_now;
+    /* update proto legend */
+    update_legend();
+
+    /* Now update info windows */
+    update_info_windows(NULL);
+
+    update_pcap_stats_text(canvas);
+  }
 
   /* Force redraw */
   while (gtk_events_pending())
     gtk_main_iteration();
 
-  gettimeofday(&appdata.gui_now, NULL);
-  last_refresh_time = appdata.gui_now;
-
   already_updating = FALSE;
 
   if (stop_requested)
     gui_stop_capture();
-
-  return TRUE;                  /* Keep on calling this function */
-}                               /* update_diagram */
+}
 
 static void purge_expired_legend_protocol(GtkWidget *widget, gpointer data)
 {
@@ -986,6 +1002,7 @@ static gint canvas_node_update(node_id_t *node_id, canvas_node_t *canvas_node,
   clock_t end;
   gdouble cpu_time_used;
   char *nametmp = NULL;
+  const gchar *main_prot;
 
   node = nodes_catalog_find(node_id);
 
@@ -1036,8 +1053,9 @@ static gint canvas_node_update(node_id_t *node_id, canvas_node_t *canvas_node,
   if (node_size > MAX_NODE_SIZE)
     node_size = MAX_NODE_SIZE;
 
-  if (node->main_prot[pref.stack_level]) {
-    canvas_node->color = *protohash_color(node->main_prot[pref.stack_level]);
+  main_prot = traffic_stats_most_used_proto(&node->node_stats, pref.stack_level);
+  if (main_prot) {
+    canvas_node->color = *protohash_color(main_prot);
 
     g_object_set(G_OBJECT(canvas_node->node_item),
                  "radius-x", node_size / 2,
@@ -1117,8 +1135,9 @@ static gboolean display_node(node_t *node)
  * which will be displayed in the diagram */
 static void limit_nodes(void)
 {
-  static GTree *ordered_nodes = NULL;
-  static guint limit;
+  GTree *ordered_nodes = NULL;
+  guint limit;
+
   displayed_nodes = 0;          /* We'll increment for each node we don't
                                  * limit */
 
@@ -1135,7 +1154,6 @@ static void limit_nodes(void)
   g_tree_foreach(ordered_nodes, (GTraverseFunc)check_ordered_node,
                  &limit);
   g_tree_destroy(ordered_nodes);
-  ordered_nodes = NULL;
 }                               /* limit_nodes */
 
 static gint add_ordered_node(node_id_t *node_id, canvas_node_t *node,
@@ -1481,7 +1499,7 @@ static gint check_new_link(link_id_t *link_id, link_t *link, GooCanvas *canvas)
   }
 
   return FALSE;
-}                               /* check_new_link */
+}
 
 
 /* - calls update_links, so that the related link updates its average
@@ -1495,6 +1513,7 @@ static gint canvas_link_update(link_id_t *link_id, canvas_link_t *canvas_link,
   const canvas_node_t *canvas_src;
   GdkRGBA scaledColor;
   double xs, ys, xd, yd, scale;
+  const gchar *main_prot;
 
   /* We used to run update_link here, but that was a major performance penalty,
    * and now it is done in update_diagram */
@@ -1526,13 +1545,22 @@ static gint canvas_link_update(link_id_t *link_id, canvas_link_t *canvas_link,
 
   /* What if there never is a protocol?
    * I have to initialize canvas_link->color to a known value */
-  if (link->main_prot[pref.stack_level]) {
+  main_prot = traffic_stats_most_used_proto(&link->link_stats, pref.stack_level);
+  if (main_prot) {
     double diffms;
-    canvas_link->color = *protohash_color(link->main_prot[pref.stack_level]);
+    double ratio;
+    canvas_link->color = *protohash_color(main_prot);
 
-    /* scale color down to 10% at link timeout */
     diffms = subtract_times_ms(&appdata.now, &link->link_stats.stats.last_time);
-    scale = pow(0.10, diffms / pref.gui_link_timeout_time);
+    ratio = diffms / pref.averaging_time;
+    scale = pow(0.5, ratio);
+    if (scale < 0.1) {
+      /* too dark, just hide */
+      goo_canvas_item_hide(canvas_link->src_item);
+      goo_canvas_item_hide(canvas_link->dst_item);
+      return FALSE;
+    }
+
 
     scaledColor.red = scale * canvas_link->color.red;
     scaledColor.green = scale * canvas_link->color.green;
@@ -1560,7 +1588,7 @@ static gint canvas_link_update(link_id_t *link_id, canvas_link_t *canvas_link,
                     canvas_link->dst_item);
 
   return FALSE;
-}                               /* update_canvas_links */
+} 
 
 /* given the src and dst node centers, plus a size, draws a triangle in the
  * specified color on the provided canvas item*/
@@ -1669,8 +1697,8 @@ static gdouble get_link_size(const basic_stats_t *link_stats)
         result = sqrt(data + 1);
         break;
   }
-  return 1.0 + pref.node_radius_multiplier * pref.link_node_ratio * result;
-}                               /* get_link_size */
+  return 1.0 + pref.node_radius_multiplier * result;
+}
 
 
 
@@ -1682,6 +1710,7 @@ static gboolean link_item_event(GooCanvasItem *item,
                                 canvas_link_t *canvas_link)
 {
   gchar *str;
+  const gchar *main_prot = NULL;
   const link_t *link = NULL;
 
   switch (event->type)
@@ -1697,9 +1726,10 @@ static gboolean link_item_event(GooCanvasItem *item,
       case GDK_ENTER_NOTIFY:
         if (canvas_link)
           link = links_catalog_find(&canvas_link->canvas_link_id);
-        if (link && link->main_prot[pref.stack_level])
-          str = g_strdup_printf(_("Link main protocol: %s"),
-                                link->main_prot[pref.stack_level]);
+        if (link)
+          main_prot = traffic_stats_most_used_proto(&link->link_stats, pref.stack_level);
+        if (main_prot)
+          str = g_strdup_printf(_("Link main protocol: %s"), main_prot);
         else
           str = g_strdup_printf(_("Link main protocol: unknown"));
         gtk_statusbar_push(appdata.statusbar, 1, str);
@@ -1854,3 +1884,18 @@ gboolean update_diagram_callback(gpointer data)
   update_diagram(gcanvas_);
   return TRUE;
 }
+
+gboolean refresh_diagram(void)
+{
+  GtkAllocation windowsize;
+  /* Simulate a window resize */
+  g_my_debug("repaint diagram requested");
+  gtk_widget_get_allocation(GTK_WIDGET(garea_), &windowsize);
+  goo_canvas_set_bounds(gcanvas_,
+                        -windowsize.width/2, -windowsize.height/2,
+                        windowsize.width/2, windowsize.height/2);
+  redraw_canvas_background(gcanvas_);
+  diagram_reposition(gcanvas_);
+  diagram_update_links(gcanvas_);
+  return TRUE;
+}
