@@ -16,7 +16,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 
 #include <math.h>
@@ -305,6 +305,8 @@ static gint protocols_table_compare(GtkTreeModel *gs, GtkTreeIter *a, GtkTreeIte
   gdouble diffms;
   gint idcol;
   GtkSortType order;
+  const port_service_t *ps1, *ps2;
+  uint port1, port2;
 
   /* reads the proto ptr from 6th columns */
   const protocol_list_item_t *prot1, *prot2;
@@ -320,11 +322,28 @@ static gint protocols_table_compare(GtkTreeModel *gs, GtkTreeIter *a, GtkTreeIte
   switch (idcol)
   {
       case PROTO_COLUMN_COLOR: /* color */
-      case PROTO_COLUMN_PORT: /* port */
       case PROTO_COLUMN_NAME: /* name */
       default:
         ret = strcmp(prot1->name, prot2->name);
         break;
+      case PROTO_COLUMN_PORT: /* port */
+        ps1 = services_port_find(prot1->name);
+        ps2 = services_port_find(prot2->name);
+        if (ps1)
+          port1 = ps1->port;
+        else
+          port1 = 0;		/* Use port: 0 for sorting if no port value is available */
+        if (ps2)
+          port2 = ps2->port;
+        else
+          port2 = 0;
+        if (port1 == port2)
+          ret = 0;
+        else if (port1 < port2)
+          ret = -1;
+        else
+          ret = 1;
+        break;
       case PROTO_COLUMN_INSTANTANEOUS:
         t1 = prot1->rowstats.average;
         t2 = prot2->rowstats.average;
@@ -682,18 +701,30 @@ static gint node_info_window_compare(gconstpointer a, gconstpointer b)
 
 gboolean update_info_windows(gpointer dummy)
 {
+  static struct timeval last_update_time = {
+    0, 0
+  };
   capstatus_t status;
+  double diffms;
+  struct timeval nowtime;
 
   status = get_capture_status();
-  if (status != PLAY && status != STOP)
-    return TRUE;
+  if (status == PAUSE)
+    return G_SOURCE_CONTINUE;
+
+  /* Update info windows at most twice a second */
+  gettimeofday(&nowtime, NULL);
+  diffms = subtract_times_ms(&nowtime, &last_update_time);
+  if (diffms >= 0 && (diffms < 500 || diffms < pref.refresh_period))
+    return G_SOURCE_CONTINUE;
 
   update_protocols_window();
   update_stats_info_windows();
   update_prot_info_windows();
   nodes_wnd_update();
 
-  return G_SOURCE_CONTINUE;     /* Keep on calling this function */
+  gettimeofday(&last_update_time, NULL);
+  return G_SOURCE_CONTINUE;
 }
 
 /* It's called when a node info window is closed by the user
@@ -718,21 +749,9 @@ void update_stats_info_windows(void)
   GList *list_item;
   GtkWidget *window;
   capstatus_t status;
-  double diffms;
-  static struct timeval last_update_time = {
-    0, 0
-  };
 
   status = get_capture_status();
 
-  /* Update info windows at most twice a second */
-  diffms = subtract_times_ms(&appdata.now, &last_update_time);
-  if (pref.refresh_period < 500)
-    if (diffms >= 0 && diffms < 500)
-      return;
-
-
-
   list_item = stats_info_windows;
   while (list_item) {
     window = list_item->data;
@@ -755,8 +774,6 @@ void update_stats_info_windows(void)
       list_item = list_item->next;
     }
   }
-
-  last_update_time = appdata.now;
 }                               /* update_stats_info_windows */
 
 
