@@ -16,7 +16,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 
 #ifdef HAVE_UNISTD_H
@@ -89,6 +89,7 @@ int main(int argc, char *argv[])
   gchar *position_file_path = NULL;
   gchar *cl_privdrop_user = NULL;
   poptContext poptcon;
+  gboolean no_obsolete;
 
   struct poptOption optionsTable[] = {
     {"diagram-only", 'd', POPT_ARG_NONE, &(pref.diagram_only), 0,
@@ -123,6 +124,8 @@ int main(int argc, char *argv[])
      N_("<delay>")},
     {"glade-file", 0, POPT_ARG_STRING, &(cl_glade_file), 0,
      N_("uses the named libglade file for widgets"), N_("<glade file>")},
+    {"no-display", 0, POPT_ARG_NONE, &(pref.headless), 0,
+     N_("calculate statistics, but don't display nodes. If replaying, exit at end [cli only]"), NULL},
     {"relinquish-privileges", 'Z', POPT_ARG_STRING, &cl_privdrop_user, 0,
      N_("run as the given user"), N_("<username>")},
 
@@ -137,6 +140,11 @@ int main(int argc, char *argv[])
 
   appdata_init(&appdata);
 
+  /* Load saved preferences */
+  no_obsolete = load_config(&pref);
+  protohash_read_prefvect(pref.colors);
+  centered_node_speclist = parse_nodeset_spec_list(pref.centered_nodes);
+
   /* Command line */
   poptcon = poptGetContext("Etherape", argc, (const char * *)argv, optionsTable, 0);
   while (poptGetNextOpt(poptcon) > 0)
@@ -161,10 +169,6 @@ int main(int argc, char *argv[])
     return 1;
   }
 
-  /* Load saved preferences */
-  load_config(&pref);
-  protohash_read_prefvect(pref.colors);
-  centered_node_speclist = parse_nodeset_spec_list(pref.centered_nodes);
 
   pref.name_res = !cl_numeric;
 
@@ -299,6 +303,16 @@ int main(int argc, char *argv[])
 
   gui_start_capture();
 
+  if (!no_obsolete) {
+    GtkWidget *dialog;
+    dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+                                          GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
+                                          _("Obsolete setting found.\n"
+                                            "Please review and save your preferences to upgrade"));
+    gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
+  }
+
   /* MAIN LOOP */
   gtk_main();
 
@@ -397,11 +411,13 @@ static GPtrArray *parse_position_file(const gchar *path)
  * but makes finding memory leaks much easier. */
 static void free_static_data(void)
 {
+  cleanup_capture();
   protohash_clear();
   ipcache_clear();
   services_clear();
   cleanup_eth_resolv();
   cleanup_names();
+  cleanup_diagram();
   appdata_free(&appdata);
 }
 
@@ -475,7 +491,6 @@ static void install_handlers(void)
  */
 void cleanup(int signum)
 {
-  cleanup_capture();
   free_static_data();
   exit(0);
 }
