@@ -16,7 +16,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 
 #include <string.h>
@@ -147,7 +147,7 @@ void on_export_activate(GtkMenuItem *menuitem, gpointer user_data)
     appdata.export_file = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
     gtk_widget_destroy(dialog);
 
-    dump_xml(appdata.export_file);
+    dump_xml(appdata.export_file, appdata.n_packets);
   }
   else
     gtk_widget_destroy(dialog);
@@ -259,6 +259,11 @@ void on_stop_menuitem_activate(GtkMenuItem *menuitem, gpointer user_data)
 }                               /* on_stop_menuitem_activate */
 
 
+void on_refresh_menuitem_activate(GtkMenuItem *menuitem, gpointer user_data)
+{
+  refresh_diagram();
+}                               /* on_refresh_menuitem_activate */
+
 
 /* View menu */
 
