@@ -17,7 +17,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 
 #include <gtk/gtk.h>
@@ -93,29 +93,19 @@ void initialize_pref_controls(void)
                            log(pref.node_radius_multiplier) / log(10));
   g_signal_emit_by_name(G_OBJECT(gtk_range_get_adjustment(GTK_RANGE(widget))),
                         "changed");
-  widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "link_width_slider"));
-  gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(widget)),
-                           pref.link_node_ratio);
-  g_signal_emit_by_name(G_OBJECT(gtk_range_get_adjustment(GTK_RANGE(widget))),
-                        "changed");
   widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "inner_ring_scale_slider"));
   gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(widget)),
                            pref.inner_ring_scale);
-  g_signal_emit_by_name(G_OBJECT(gtk_range_get_adjustment(GTK_RANGE(widget))),
-                        "changed");
+  g_signal_emit_by_name(G_OBJECT(gtk_range_get_adjustment(GTK_RANGE(widget))), "changed");
   spin = GTK_SPIN_BUTTON(gtk_builder_get_object(appdata.xml, "averaging_spin"));
   gtk_spin_button_set_value(spin, pref.averaging_time);
   spin = GTK_SPIN_BUTTON(gtk_builder_get_object(appdata.xml, "refresh_spin"));
   gtk_spin_button_set_value(spin, pref.refresh_period);
-  spin = GTK_SPIN_BUTTON(gtk_builder_get_object(appdata.xml, "node_to_spin"));
-  gtk_spin_button_set_value(spin, pref.node_timeout_time/MILLI);
   spin = GTK_SPIN_BUTTON(gtk_builder_get_object(appdata.xml, "gui_node_to_spin"));
   gtk_spin_button_set_value(spin, pref.gui_node_timeout_time/MILLI);
   spin = GTK_SPIN_BUTTON(gtk_builder_get_object(appdata.xml, "proto_node_to_spin"));
   gtk_spin_button_set_value(spin, pref.proto_node_timeout_time/MILLI);
 
-  spin = GTK_SPIN_BUTTON(gtk_builder_get_object(appdata.xml, "link_to_spin"));
-  gtk_spin_button_set_value(spin, pref.link_timeout_time/MILLI);
   spin = GTK_SPIN_BUTTON(gtk_builder_get_object(appdata.xml, "gui_link_to_spin"));
   gtk_spin_button_set_value(spin, pref.gui_link_timeout_time/MILLI);
   spin = GTK_SPIN_BUTTON(gtk_builder_get_object(appdata.xml, "proto_link_to_spin"));
@@ -184,10 +174,6 @@ void initialize_pref_controls(void)
   g_signal_connect(G_OBJECT(gtk_range_get_adjustment(GTK_RANGE(widget))),
                    "value_changed",
                    G_CALLBACK(on_node_radius_slider_adjustment_changed), NULL);
-  widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "link_width_slider"));
-  g_signal_connect(G_OBJECT(gtk_range_get_adjustment(GTK_RANGE(widget))),
-                   "value_changed",
-                   G_CALLBACK(on_link_width_slider_adjustment_changed), NULL);
   widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "inner_ring_scale_slider"));
   g_signal_connect(G_OBJECT(gtk_range_get_adjustment(GTK_RANGE(widget))),
                    "value_changed",
@@ -201,10 +187,6 @@ void initialize_pref_controls(void)
                    "value_changed",
                    G_CALLBACK(on_refresh_spin_adjustment_changed),
                    canvas_widget());
-  widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "node_to_spin"));
-  g_signal_connect(G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(widget))),
-                   "value_changed",
-                   G_CALLBACK(on_node_to_spin_adjustment_changed), NULL);
   widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "gui_node_to_spin"));
   g_signal_connect(G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(widget))),
                    "value_changed",
@@ -213,10 +195,6 @@ void initialize_pref_controls(void)
   g_signal_connect(G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(widget))),
                    "value_changed",
                    G_CALLBACK(on_proto_node_to_spin_adjustment_changed), NULL);
-  widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "link_to_spin"));
-  g_signal_connect(G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(widget))),
-                   "value_changed",
-                   G_CALLBACK(on_link_to_spin_adjustment_changed), NULL);
   widget = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "gui_link_to_spin"));
   g_signal_connect(G_OBJECT(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(widget))),
                    "value_changed",
@@ -279,15 +257,6 @@ void on_node_radius_slider_adjustment_changed(GtkAdjustment *adj)
         value, pref.node_radius_multiplier);
 }
 
-void on_link_width_slider_adjustment_changed(GtkAdjustment *adj)
-{
-  double value = gtk_adjustment_get_value(adj);
-  pref.link_node_ratio = value;
-  g_log(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
-        _("Adjustment value: %g. Link-node ratio %g"),
-        value, pref.link_node_ratio);
-}
-
 void on_inner_ring_scale_slider_adjustment_changed(GtkAdjustment *adj)
 {
   double value = gtk_adjustment_get_value(adj);
@@ -308,11 +277,6 @@ void on_refresh_spin_adjustment_changed(GtkAdjustment *adj, GtkWidget *canvas)
   change_refresh_period(gtk_adjustment_get_value(adj));
 }
 
-void on_node_to_spin_adjustment_changed(GtkAdjustment *adj)
-{
-  pref.node_timeout_time = gtk_adjustment_get_value(adj)*MILLI; /* value in ms */
-}
-
 void on_gui_node_to_spin_adjustment_changed(GtkAdjustment *adj)
 {
   pref.gui_node_timeout_time = gtk_adjustment_get_value(adj)*MILLI;     /* value in ms */
@@ -323,11 +287,6 @@ void on_proto_node_to_spin_adjustment_changed(GtkAdjustment *adj)
   pref.proto_node_timeout_time = gtk_adjustment_get_value(adj)*MILLI;   /* value in ms */
 }
 
-void on_link_to_spin_adjustment_changed(GtkAdjustment *adj)
-{
-  pref.link_timeout_time = gtk_adjustment_get_value(adj)*MILLI; /* value in ms */
-}
-
 void on_gui_link_to_spin_adjustment_changed(GtkAdjustment *adj)
 {
   pref.gui_link_timeout_time = gtk_adjustment_get_value(adj)*MILLI;     /* value in ms */
@@ -493,7 +452,6 @@ typedef struct _EATreePos
   GtkListStore *gs;
 } EATreePos;
 
-
 /* fill ep with the listore for the color treeview, creating it if necessary
    Returns FALSE if something goes wrong
 */
@@ -531,23 +489,15 @@ static gboolean get_color_store(EATreePos *ep)
   return TRUE;
 }
 
-
-void on_color_add_button_clicked(GtkButton *button, gpointer user_data)
+void on_color_edit_dialog_show(GtkWidget *wdg, gpointer user_data)
 {
-  GtkWidget *dlg =
-    GTK_WIDGET(gtk_builder_get_object(appdata.xml, "colorselectiondialog"));
-  g_object_set_data(G_OBJECT(dlg), "isadd", GINT_TO_POINTER(TRUE));
-  gtk_widget_show(dlg);
-}                               /* on_color_add_button_clicked */
-
-void on_color_change_button_clicked(GtkButton *button, gpointer user_data)
-{
-  GtkTreePath *gpath;
-  GtkTreeViewColumn *gcol;
-  GtkTreeIter it;
+  gchar *protocol_string;
   GdkRGBA *color;
-  GtkColorSelectionDialog *dlg;
-  GtkColorSelection *csel;
+  GtkTreePath *gpath = NULL;
+  GtkTreeViewColumn *gcol = NULL;
+  GtkTreeIter it;
+  GtkComboBox *cbox;
+  GtkColorChooser *chos;
   EATreePos ep;
 
   if (!get_color_store(&ep))
@@ -555,25 +505,41 @@ void on_color_change_button_clicked(GtkButton *button, gpointer user_data)
 
   /* gets the row (path) at cursor */
   gtk_tree_view_get_cursor(ep.gv, &gpath, &gcol);
-  if (!gpath)
-    return; /* no row selected */
+  if (gpath) {
+    /* row selected */
+    if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(ep.gs), &it, gpath))
+      return; /* path not found */
 
-  /* get iterator from path */
-  if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(ep.gs), &it, gpath))
-    return;
+    gtk_tree_model_get(GTK_TREE_MODEL(ep.gs), &it, 1, &color, 2, &protocol_string, -1);
 
-  gtk_tree_model_get(GTK_TREE_MODEL(ep.gs), &it, 1, &color, -1);
+    cbox = GTK_COMBO_BOX(gtk_builder_get_object(appdata.xml, "protocol_entry"));
+    cbox_add_select(cbox, protocol_string);
 
-  dlg = GTK_COLOR_SELECTION_DIALOG(
-    gtk_builder_get_object(appdata.xml, "colorselectiondialog"));
+    chos = GTK_COLOR_CHOOSER(gtk_builder_get_object(appdata.xml, "color_entry"));
+    gtk_color_chooser_set_rgba(chos, color);
 
-  csel = GTK_COLOR_SELECTION(gtk_color_selection_dialog_get_color_selection(dlg));
-  gtk_color_selection_set_current_rgba(csel, color);
-  gtk_color_selection_set_previous_rgba(csel, color);
+    g_free(protocol_string);
+  }
+}
 
-  g_object_set_data(G_OBJECT(dlg), "isadd", GINT_TO_POINTER(FALSE));
-  gtk_widget_show(GTK_WIDGET(dlg));
-}                               /* on_color_change_button_clicked */
+
+void on_color_add_button_clicked(GtkButton *button, gpointer user_data)
+{
+  GtkWidget *dlg = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "color_edit_dialog"));
+  if (dlg) {
+    g_object_set_data(G_OBJECT(dlg), "isadd", GINT_TO_POINTER(TRUE));
+    gtk_widget_show(dlg);
+  }
+} 
+
+void on_color_edit_button_clicked(GtkButton *button, gpointer user_data)
+{
+  GtkWidget *dlg = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "color_edit_dialog"));
+  if (dlg) {
+    g_object_set_data(G_OBJECT(dlg), "isadd", GINT_TO_POINTER(FALSE));
+    gtk_widget_show(dlg);
+  }
+} 
 
 void on_color_remove_button_clicked(GtkButton *button, gpointer user_data)
 {
@@ -593,21 +559,16 @@ void on_color_remove_button_clicked(GtkButton *button, gpointer user_data)
   if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(ep.gs), &it, gpath))
     return; /* path not found */
 
-#if GTK_CHECK_VERSION(2, 2, 0)
   if (gtk_list_store_remove(ep.gs, &it)) {
     /* iterator still valid, selects current pos */
     gpath = gtk_tree_model_get_path(GTK_TREE_MODEL(ep.gs), &it);
     gtk_tree_view_set_cursor(ep.gv, gpath, NULL, 0);
     gtk_tree_path_free(gpath);
   }
-#else
-  /* gtk < 2.2 had gtk_list_store_remove void */
-  gtk_list_store_remove(ep.gs, &it);
-#endif
 
   colors_changed = TRUE;
   color_list_to_pref();
-}                               /* on_color_remove_button_clicked */
+}
 
 void on_colordiag_ok_clicked(GtkButton *button, gpointer user_data)
 {
@@ -660,82 +621,72 @@ void on_colordiag_ok_clicked(GtkButton *button, gpointer user_data)
 
   color_list_to_pref();
   colors_changed = TRUE;
-}                               /* on_colordiag_ok_clicked */
-
-
-
-void on_protocol_edit_button_clicked(GtkButton *button, gpointer user_data)
-{
-  GtkWidget *protocol_edit_dialog = NULL;
-  protocol_edit_dialog = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "protocol_edit_dialog"));
-  gtk_widget_show(protocol_edit_dialog);
-}                               /* on_protocol_edit_button_clicked */
-
-void on_protocol_edit_dialog_show(GtkWidget *wdg, gpointer user_data)
-{
-  gchar *protocol_string;
-  GtkTreePath *gpath = NULL;
-  GtkTreeViewColumn *gcol = NULL;
-  GtkTreeIter it;
-  GtkComboBox *cbox;
-  EATreePos ep;
-
-  if (!get_color_store(&ep))
-    return;
-
-  /* gets the row (path) at cursor */
-  gtk_tree_view_get_cursor(ep.gv, &gpath, &gcol);
-  if (!gpath)
-    return; /* no row selected */
-
-  if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(ep.gs), &it, gpath))
-    return; /* path not found */
-
-  gtk_tree_model_get(GTK_TREE_MODEL(ep.gs), &it, 2, &protocol_string, -1);
-
-  cbox = GTK_COMBO_BOX(gtk_builder_get_object(appdata.xml, "protocol_entry"));
-  cbox_add_select(cbox, protocol_string);
-
-  g_free(protocol_string);
 }
 
-
-void on_protocol_edit_ok_clicked(GtkButton *button, gpointer user_data)
+void on_color_edit_ok_clicked(GtkButton *button, gpointer user_data)
 {
   const gchar *combo_string;
   gchar *proto_string;
+  GdkRGBA color;
   GtkTreePath *gpath = NULL;
   GtkTreeViewColumn *gcol = NULL;
   GtkTreeIter it;
   GtkComboBox *cbox;
+  GtkColorChooser *chos;
+  gint isadd;
   EATreePos ep;
+  GtkWidget *dlg;
+  
+  dlg = GTK_WIDGET(gtk_builder_get_object(appdata.xml, "color_edit_dialog"));
+  if (!dlg)
+    return;
+
   if (!get_color_store(&ep))
     return;
 
-  /* gets the row (path) at cursor */
-  gtk_tree_view_get_cursor(ep.gv, &gpath, &gcol);
-  if (!gpath)
-    return; /* no row selected */
-
-  if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(ep.gs), &it, gpath))
-    return; /* path not found */
+  /* modify or add ? */
+  isadd = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(dlg), "isadd"));
 
   cbox = GTK_COMBO_BOX(gtk_builder_get_object(appdata.xml, "protocol_entry"));
   combo_string = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(cbox))));
   proto_string = g_utf8_strup(g_strdup(combo_string), -1);
   proto_string = remove_spaces(proto_string);
-
   cbox_add_select(cbox, proto_string);
-  gtk_list_store_set(ep.gs, &it, 2, proto_string, -1);
+
+  chos = GTK_COLOR_CHOOSER(gtk_builder_get_object(appdata.xml, "color_entry"));
+  gtk_color_chooser_get_rgba(chos, &color);
+
+  /* gets the row (path) at cursor */
+  gtk_tree_view_get_cursor(ep.gv, &gpath, &gcol);
+  if (isadd) {
+    if (gpath) {
+      /* row sel, add/change color */
+      GtkTreeIter itsibling;
+      if (!gtk_tree_model_get_iter(GTK_TREE_MODEL(ep.gs), &itsibling, gpath))
+        return; /* path not found */
+      gtk_list_store_insert_before(ep.gs, &it, &itsibling);
+    }
+    else
+      gtk_list_store_append(ep.gs, &it); /* no row selected, append */
+  }
+  else {
+    if (!gpath ||
+        !gtk_tree_model_get_iter(GTK_TREE_MODEL(ep.gs), &it, gpath))
+      return; /* path or node not found */
+  }
+
+  gtk_list_store_set(ep.gs, &it, 
+                     0, COLSPACES, 
+                     1, &color, 
+                     2, proto_string, 
+                     -1);
 
   g_free(proto_string);
-  gtk_widget_hide(GTK_WIDGET(gtk_builder_get_object(appdata.xml, "protocol_edit_dialog")));
+  gtk_widget_hide(dlg);
 
   colors_changed = TRUE;
   color_list_to_pref();
-}                               /* on_protocol_edit_ok_clicked */
-
-
+}
 
 static void pref_to_color_list(void)
 {
