@@ -16,7 +16,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "../config.h"
+#include "config.h"
 #endif
 #include <gtk/gtk.h>
 #include "preferences.h"
@@ -142,6 +142,7 @@ static const struct preference preferences[] = {
   MKPREF(name_res, bool, TRUE),
   MKPREF(diagram_only, bool, FALSE),
   MKPREF(group_unk, bool, TRUE),
+  MKPREF(headless, bool, FALSE),
   MKPREF(centered_nodes, string, ""),
 
   MKPREF(text_color, string, "#ffff00"),
@@ -161,19 +162,16 @@ static const struct preference preferences[] = {
 
   MKPREF(inner_ring_scale, double, 0.5),
   MKPREF(node_radius_multiplier, double, 0.0005),
-  MKPREF(link_node_ratio, double, 1.0),
   MKPREF(node_size_variable, int, INST_OUTBOUND),
   MKPREF(size_mode, int, LINEAR),
 
-  MKPREF(node_timeout_time, double, 120000.0),
   MKPREF(gui_node_timeout_time, double, 60000.0),
-  MKPREF(proto_node_timeout_time, double, 0),
+  MKPREF(proto_node_timeout_time, double, 120000.0),
 
-  MKPREF(link_timeout_time, double, 20000.0),
-  MKPREF(gui_link_timeout_time, double, 20000.0),
-  MKPREF(proto_link_timeout_time, double, 600000.0),
+  MKPREF(gui_link_timeout_time, double, 3000.0),
+  MKPREF(proto_link_timeout_time, double, 200000.0),
 
-  MKPREF(proto_timeout_time, double, 60000.0),
+  MKPREF(proto_timeout_time, double, 600000.0),
   MKPREF(refresh_period, int, 100),
   MKPREF(averaging_time, double, 2000.0),
 
@@ -228,13 +226,42 @@ static void default_config(struct pref_struct *p)
   }
 }
 
-/* loads configuration from .gnome/Etherape */
-void load_config(struct pref_struct *p)
+/* checks for obsolete configuration */
+static gboolean check_obsolete_config(GKeyFile *gkey)
+{
+  /* list of obsolete keys to check */
+  static const char *older_configs[] = { 
+                                        "link_node_ratio",
+                                        "link_timeout_time",
+                                        "node_timeout_time",
+                                        /* must be the last element */
+                                        NULL  
+                                        };
+  int i;
+
+  for (i = 0; older_configs[i]; ++i) {
+    gchar *tmp;
+    tmp = g_key_file_get_string(gkey, pref_group, older_configs[i], NULL);
+    if (!tmp) 
+      continue;  /* ok, key not found */
+    
+    /* obsolete key found - error */
+    g_free(tmp);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
+
+/* loads configuration from .gnome/Etherape - returns FALSE if config contains obsolete items */
+gboolean load_config(struct pref_struct *p)
 {
   gchar *pref_file;
   GKeyFile *gkey;
   int i;
   void *addr;
+  gboolean no_obsolete;
 
   /* first set up defaults */
   default_config(p);
@@ -249,11 +276,14 @@ void load_config(struct pref_struct *p)
     pref_file = old_config_file_name();
     if (!g_key_file_load_from_file(gkey, pref_file, G_KEY_FILE_NONE, NULL)) {
       g_free(pref_file);
-      return;
+      return TRUE;
     }
   }
   g_free(pref_file);
 
+  /* first check for obsolete settings */
+  no_obsolete = check_obsolete_config(gkey);
+
   for (i = 0; i < NUM_PREFS; i++) {
     addr = (char *)p + preferences[i].offset;
     switch (preferences[i].type)
@@ -283,6 +313,7 @@ void load_config(struct pref_struct *p)
   p->colors = protohash_compact(p->colors);
 
   g_key_file_free(gkey);
+  return no_obsolete;
 }
 
 /* saves configuration to .gnome/Etherape */
