@@ -58,6 +58,7 @@
 #define INET6_ADDRSTRLEN  46
 #endif
 #include <locale.h>
+#include <assert.h>
 
 #include "appdata.h"
 #include "util.h"
@@ -69,6 +70,8 @@ char *safe_strncpy(char *dst, const char *src, size_t maxlen)
 {
   if (maxlen < 1)
     return dst;
+  assert(src);
+  assert(dst);
   strncpy(dst, src, maxlen - 1);       /* no need to copy that last char */
   dst[maxlen - 1] = '\0';
   return dst;
@@ -77,10 +80,12 @@ char *safe_strncpy(char *dst, const char *src, size_t maxlen)
 /* safe strncat */
 char *safe_strncat(char *dst, const char *src, size_t maxlen)
 {
+  assert(dst);
+  assert(src);
   size_t lendst = strlen(dst);
   if (lendst >= maxlen)
     return dst; /* already full, nothing to do */
-  strncat(dst, src, maxlen - strlen(dst));
+  strncat(dst, src, maxlen - lendst);
   dst[maxlen - 1] = '\0';
   return dst;
 }
@@ -338,6 +343,41 @@ gchar *timeval_to_str(struct timeval last_heard)
  * xml helpers
  *
  *************************************************/
+gchar *xmlescape(const gchar *str)
+{
+  gchar *escaped;
+  gchar *cur;
+
+  size_t maxlen = strlen(str) * 4 + 8;
+  g_assert(maxlen < 16392);     //  just to avoid abnormal strings
+
+  escaped = g_malloc(maxlen);  // allow worst case space
+  cur = escaped;
+  while (*str != '\0') {
+    switch (*str) {
+      case '&':
+        *cur++ = '&';
+        *cur++ = 'a';
+        *cur++ = 'm';
+        *cur++ = 'p';
+        *cur++ = ';';
+        break;
+      case '<':
+        *cur++ = '&';
+        *cur++ = 'l';
+        *cur++ = 't';
+        *cur++ = ';';
+        break;
+      default:
+        *cur++ = *str;
+        break;
+    }
+    ++str;
+  }
+  *cur = '\0';
+  return escaped; 
+}
+
 
 /* returns a new string containing the named tag */
 gchar *xmltag(const gchar *name, const gchar *fmt, ...)
@@ -353,4 +393,20 @@ gchar *xmltag(const gchar *name, const gchar *fmt, ...)
   return xml;
 }
 
+/* returns a new string containing the named tag with the string data escaped */
+gchar *xmltag_escaped(const gchar *name, const gchar *fmt, ...)
+{
+  gchar *msg;
+  gchar *escaped;
+  gchar *xml;
+  va_list ap;
+  va_start(ap, fmt);
+  msg = g_strdup_vprintf(fmt, ap);
+  va_end(ap);
+  escaped = xmlescape(msg);
+  xml = g_strdup_printf("<%s>%s</%s>\n", name, escaped, name);
+  g_free(escaped);
+  g_free(msg);
+  return xml;
+}
 
